from swampdragon.route_handler import BaseModelRouter
from swampdragon.serializers.model_serializer import ModelSerializer
from swampdragon.testing.dragon_testcase import DragonTestCase
from .models import TextModel


class Serializer(ModelSerializer):
    class Meta:
        model = TextModel


class Router(BaseModelRouter):
    model = TextModel
    serializer_class = Serializer

    def get_query_set(self):
        return self.model.objects.all()


class TestBaseModelRouter(DragonTestCase):
    def test_get_query_set(self):
        TextModel.objects.create(text='This is a text model')

        router = Router(self.connection)
        with self.assertNumQueries(1):
            for i in range(2):
                router.get_list()



#!/usr/bin/env python
import os
import sys


from swampdragon.swampdragon_server import run_server

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "<project>.settings")

host_port = sys.argv[1] if len(sys.argv) > 1 else None

run_server(host_port=host_port)



# From https://raw.githubusercontent.com/tomchristie/django-rest-framework/master/tests/conftest.py
def pytest_configure():
    from django.conf import settings

    settings.configure(
        DEBUG_PROPAGATE_EXCEPTIONS=True,
        DATABASES={'default': {'ENGINE': 'django.db.backends.sqlite3',
                               'NAME': ':memory:'}},
        SECRET_KEY='not very secret in tests',
        USE_I18N=True,
        USE_L10N=True,
        STATIC_URL='/static/',
        ROOT_URLCONF='tests.urls',
        TEMPLATE_LOADERS=(
            'django.template.loaders.filesystem.Loader',
            'django.template.loaders.app_directories.Loader',
        ),
        MIDDLEWARE_CLASSES=(
            'django.middleware.common.CommonMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
        ),
        INSTALLED_APPS=(
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.sites',
            'django.contrib.messages',
            'django.contrib.staticfiles',

            'tests',
            'swampdragon',
        ),
        PASSWORD_HASHERS=(
            'django.contrib.auth.hashers.SHA1PasswordHasher',
            'django.contrib.auth.hashers.PBKDF2PasswordHasher',
            'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
            'django.contrib.auth.hashers.BCryptPasswordHasher',
            'django.contrib.auth.hashers.MD5PasswordHasher',
            'django.contrib.auth.hashers.CryptPasswordHasher',
        ),

        SWAMP_DRAGON_CONNECTION=('swampdragon.connections.sockjs_connection.DjangoSubscriberConnection', '/data'),
        SWAMPDRAGON_TESTMODE=True
    )

    try:
        import django
        django.setup()
    except AttributeError:
        pass



import json
import redis
from .redis_settings import get_redis_host, get_redis_port, get_redis_db

_redis_cli = None


def get_redis_cli():
    global _redis_cli
    if not _redis_cli:
        _redis_cli = redis.StrictRedis(
            host=get_redis_host(),
            port=get_redis_port(),
            db=get_redis_db()
        )
    return _redis_cli


def publish(channel, message):
    get_redis_cli().publish(channel, json.dumps(message))


def _get_channels_from_redis(base_channel):
    channels = get_redis_cli().execute_command('PUBSUB', 'channels', '{}*'.format(base_channel))
    return [c.decode() for c in channels]


def get_channels(base_channel):
    return _get_channels_from_redis(base_channel)



from .redis_session_store import RedisSessionStore
from django.conf import settings
from django.utils.importlib import import_module


session_store = None


def get_session_store():
    global session_store
    if session_store:
        return session_store

    if not hasattr(settings, 'SWAMP_DRAGON_SESSION_STORE'):
        session_store = RedisSessionStore
        return session_store
    else:
        try:
            module_name, cls_name = settings.SWAMP_DRAGON_SESSION_STORE.rsplit('.', 1)
            module = import_module(module_name)
            cls = getattr(module, cls_name)
            session_store = cls
        except:
            session_store = RedisSessionStore
        finally:
            return session_store



from django.conf import settings as django_settings


class SwampDragonSettings(object):
    def to_dict(self):
        settings = getattr(django_settings, 'SWAMP_DRAGON', {})
        settings['endpoint'] = django_settings.SWAMP_DRAGON_CONNECTION[1]
        return settings



from django.conf import settings

redis_host = None
redis_port = None
redis_db = None


def get_redis_host():
    global redis_host
    if not redis_host:
        redis_host = getattr(settings, 'SWAMP_DRAGON_REDIS_HOST', 'localhost')
    return redis_host


def get_redis_port():
    global redis_port
    if not redis_port:
        redis_port = getattr(settings, 'SWAMP_DRAGON_REDIS_PORT', 6379)
    return redis_port


def get_redis_db():
    global redis_db
    if not redis_db:
        redis_db = getattr(settings, 'SWAMP_DRAGON_REDIS_DB', 0)
    return redis_db



from django.core.management.base import BaseCommand

from swampdragon.swampdragon_server import run_server


class Command(BaseCommand):
    def handle(self, *args, **options):
        host_port = None
        if args:
            host_port = args[0]
        run_server(host_port=host_port)



import os
from django.conf import settings


_pub_sub = None


def set_test_mode():
    os.environ.setdefault('SWAMPDRAGON_TESTMODE', 'True')


def test_mode():
    if os.environ.get('SWAMPDRAGON_TESTMODE') == 'True':
        return True

    if getattr(settings, 'SWAMPDRAGON_TESTMODE', False) is True:
        return True

    return False



class BaseSessionStore(object):
    def __init__(self, connection):
        self.connection = connection
        self.keys = []

    def set(self, key, val):
        raise NotImplemented()

    def get(self, key):
        raise NotImplemented()

    def refresh_key_timeout(self, key):
        raise NotImplemented()

    def refresh_all_keys(self):
        for key in self.keys:
            self.refresh_key_timeout(key)



import django
from django.conf import settings
from django.utils.importlib import import_module
from tornado import web, ioloop
from sockjs.tornado import SockJSRouter
from swampdragon import discover_routes, load_field_deserializers, VERSION
from swampdragon.settings_provider import SettingsHandler


def _output_server(host, port):
    """
    Print info about the current instance
    of SwampDragon
    """
    print('-------- SwampDragon ------')
    print('Running SwampDragon on {}:{}'.format(host, port))
    print('DRAGON_URL: {}'.format(settings.DRAGON_URL))
    print('Version {}'.format('.'.join([str(v) for v in VERSION])))
    print('Debug: {}'.format(settings.DEBUG))
    print('Quit the server with ctrl+c')
    print('---------------------------')


def run_server(host_port=None):
    if hasattr(django, 'setup'):
        django.setup()

    HOST = getattr(settings, 'SWAMP_DRAGON_HOST', '127.0.0.1')
    PORT = getattr(settings, 'SWAMP_DRAGON_PORT', 9999)

    if host_port is not None:
        HOST = host_port.split(':')[0]
        PORT = host_port.split(':')[1]
    routers = []

    if hasattr(settings, 'SOCKJS_CLASSES'):
        raise Exception('''
--------------
The SOCKJS_CLASSES setting has been removed in favour of SWAMP_DRAGON_CONNECTION

Update your settings and add SWAMP_DRAGON_CONNECTION.
--------------
        ''')

    module_name, cls_name = settings.SWAMP_DRAGON_CONNECTION[0].rsplit('.', 1)
    module = import_module(module_name)
    cls = getattr(module, cls_name)
    channel = settings.SWAMP_DRAGON_CONNECTION[1]
    routers.append(SockJSRouter(cls, channel))

    app_settings = {
        'debug': settings.DEBUG,
    }

    urls = discover_routes()
    for router in routers:
        urls += router.urls
    urls.append(('/settings.js$', SettingsHandler))

    load_field_deserializers()

    app = web.Application(urls, **app_settings)
    app.listen(PORT, address=HOST, no_keep_alive=False)

    _output_server(host=HOST, port=PORT)

    try:
        iol = ioloop.IOLoop.instance()
        iol.start()
    except KeyboardInterrupt:
        # so you don't think you erred when ^C'ing out
        pass



from .redis_sub_provider import RedisSubProvider
from .mock_sub_provider import MockSubProvider
from swampdragon.testing import test_mode


_subscriber = None


def get_subscription_provider():
    global _subscriber
    if not _subscriber:
        if test_mode.test_mode():
            _subscriber = MockSubProvider()
        else:
            _subscriber = RedisSubProvider()
    return _subscriber



from swampdragon.route_handler import ModelRouter
from swampdragon.testing.dragon_testcase import DragonTestCase
from .models import BoolSelfPub
from .serializers import BoolSelfPubSerializer


class BoolSelfPubRouter(ModelRouter):
    serializer_class = BoolSelfPubSerializer


class TestSelfPubModel(DragonTestCase):
    def test_self_pub_model_bool_change(self):
        router = BoolSelfPubRouter(self.connection)
        router.subscribe(**{'channel': 'testchan'})
        self.assertIsNone(self.connection.last_pub)
        model = BoolSelfPub.objects.create(bool=True)
        self.assertIsNotNone(self.connection.last_pub)
        self.assertEqual(len(self.connection.published_data), 1)
        model.bool = False
        model.save()
        self.assertEqual(len(self.connection.published_data), 2)
        model.bool = True
        model.save()
        self.assertEqual(len(self.connection.published_data), 3)



try:
    from http.cookies import SimpleCookie
except ImportError:
    from Cookie import SimpleCookie

from django.conf import settings
from sockjs.tornado.session import ConnectionInfo
from swampdragon.connections.sockjs_connection import SubscriberConnection
from swampdragon.testing.dragon_testcase import DragonTestCaseAsync
import uuid


class TestSession(object):
    def __init__(self, is_open=True):
        self.session_id = uuid.uuid4().hex
        self.is_closed = is_open is False
        self.messages = []

    def send_message(self, message, binary=False):
        self.messages.append(message)

    def close(self, code=3000, message='Connection closed'):
        self.is_closed = True


class TestSameOrigin(DragonTestCaseAsync):
    def test_same_origin(self):
        settings.DRAGON_URL = self.host
        settings.SWAMP_DRAGON_SAME_ORIGIN = True

        response = self.fetch('/settings.js')
        cookie = SimpleCookie(response.headers['Set-Cookie'])
        request = ConnectionInfo('127.0.0.1', cookies=cookie, arguments={}, headers={}, path='/data/983/9cz4ridg/websocket')
        session = TestSession()
        self.connection = SubscriberConnection(session)
        self.connection.on_open(request)

        self.assertFalse(self.connection.is_closed)



from swampdragon.connections.sockjs_connection import SubscriberConnection
from swampdragon.route_handler import BaseRouter
from swampdragon.testing.dragon_testcase import DragonTestCase
from swampdragon.connections.mock_connection import TestConnection
from swampdragon.sessions.session_store import BaseSessionStore
from swampdragon.sessions import sessions
from .test_subscriber_connection import TestSession
import json


class FooRouter(BaseRouter):
    route_name = 'foo-router'
    valid_verbs = ['write_session', 'read_session']

    def write_session(self, **kwargs):
        self.connection.session_store.set('key', kwargs['value'])

    def read_session(self):
        val = self.connection.session_store.get('key')
        self.send(val)


class TestSessionStore(BaseSessionStore):
    data = {}

    def set(self, key, val):
        self.data[key] = val

    def get(self, key):
        return self.data[key]


class TestSessions(DragonTestCase):
    def setUp(self):
        self.session = TestSession()
        self.connection = SubscriberConnection(self.session)
        self.session_store = self.connection.session_store

    def test_read_from_session(self):
        val = 'test val'
        self.session_store.set('a_key', val)
        self.assertEqual(val, self.session_store.get('a_key'))

    def test_overwrite_value(self):
        val = 'test val'
        self.session_store.set('a_key', val)
        self.session_store.set('a_key', 'updated val')
        self.assertEqual('updated val', self.session_store.get('a_key'))

    def test_write_dict(self):
        data = {'a': 'val', 'b': 1}
        key = 'key'
        self.session_store.set(key, data)
        data_from_session = json.loads(self.session_store.get(key))
        self.assertDictEqual(data, data_from_session)

    def test_session_from_router(self):
        """
        Ensure that sessions are unique per connection
        """
        connection_a = TestConnection()
        connection_b = TestConnection()
        foo_router = FooRouter(connection_a)
        foo_router.write_session(**{'value': 'a value'})
        foo_router.read_session()
        self.assertEqual(connection_a.get_last_message()['data'], 'a value')

        foo_router = FooRouter(connection_b)
        foo_router.read_session()
        self.assertNotEqual(connection_b.get_last_message()['data'], 'a value')


class TestCustomSessionStore(DragonTestCase):
    def test_custom_session_store(self):
        sessions.session_store = TestSessionStore
        session_store = sessions.get_session_store()(self.connection)
        session_store.set('key', 'val')
        self.assertEqual(session_store.get('key'), 'val')



from swampdragon.pubsub_providers.base_provider import PUBACTIONS
from swampdragon.route_handler import BaseRouter
from swampdragon.testing.dragon_testcase import DragonTestCase


class FooRouter(BaseRouter):
    pass


class TestBaseRouter(DragonTestCase):
    def test_get_list(self):
        channel_data = FooRouter(self.connection).make_channel_data('client_chan', 'server_chan', action=PUBACTIONS.created)
        expected = {'local_channel': 'client_chan', 'remote_channels': 'server_chan', 'action': PUBACTIONS.created}
        self.assertDictEqual(channel_data, expected)



from swampdragon.serializers.validation import ValidationError


class SerializerMeta(object):
    def __init__(self, options):
        self.publish_fields = getattr(options, 'publish_fields', None)

        if isinstance(self.publish_fields, str):
            self.publish_fields = (self.publish_fields, )

        self.update_fields = getattr(options, 'update_fields', ())
        if isinstance(self.update_fields, str):
            self.update_fields = (self.update_fields, )


class Serializer(object):
    def __init__(self, data=None, initial=None):
        if data and not isinstance(data, dict):
            raise Exception('data needs to be a dictionary')
        self.opts = SerializerMeta(self.Meta)
        self.data = data
        self.clean_data = {}
        self.initial = initial or {}
        self.errors = {}

    def save(self):
        self.deserialize()
        return self.clean_data

    def deserialize(self):
        for key, val in self.initial.items():
            self.clean_data[key] = val

        # Deserialize base fields
        for key, val in self.data.items():
            if key not in self.opts.update_fields:
                continue
            try:
                self.validate_field(key, val, self.data)
                self._deserialize_field(key, val)
            except ValidationError as err:
                self.errors.update(err.get_error_dict())

    def validate_field(self, field, value, data):
        validation_name = 'validate_{}'.format(field)
        if hasattr(self, validation_name):
            validator = getattr(self, validation_name)
            validator(value)
        return None

    def _get_custom_field_serializers(self):
        """
        Get all custom serializer functions.
        If this function has a serializer attached to it, include that
        """
        functions = [(
            getattr(self, f),
            f.replace('serialize_', '')
        ) for f in dir(self) if f.startswith('serialize_')]
        return functions



from swampdragon.pubsub_providers import redis_publisher, mock_publisher
from swampdragon.testing import test_mode

_publisher = None


def get_publisher():
    global _publisher
    if not _publisher:
        if test_mode.test_mode():
            _publisher = mock_publisher.MockPublisher()
        else:
            _publisher = redis_publisher
    return _publisher



from django.conf import settings
from sockjs.tornado import SockJSConnection
from tornado.ioloop import PeriodicCallback
from ..pubsub_providers.subscriber_factory import get_subscription_provider
from .. import route_handler
from ..sessions.sessions import get_session_store
from ..same_origin import set_origin_connection, test_origin
import json


session_store = get_session_store()
heartbeat_frequency = None
heartbeat_enabled = None


def get_heartbeat_frequency():
    global heartbeat_frequency
    if not heartbeat_frequency:
        heartbeat_frequency = getattr(settings, 'SWAMP_DRAGON_HEARTBEAT_FREQUENCY', 1000 * 60 * 20)  # Default to 20 minutes
    return heartbeat_frequency


def is_heartbeat_enabled():
    global heartbeat_enabled
    if not heartbeat_enabled:
        heartbeat_enabled = getattr(settings, 'SWAMP_DRAGON_HEARTBEAT_ENABLED', False)
    return heartbeat_enabled


class ConnectionMixin(object):
    def to_json(self, data):
        """
        If the message is a dict, return it.
        If it's a string try to decode it into json,
        otherwise assume it's not a dictionary but a simple text message
        """
        if isinstance(data, dict):
            return data
        try:
            return json.loads(data)
        except:
            return {'message': data}


class SubscriberConnection(ConnectionMixin, SockJSConnection):
    pub_sub = None

    def __init__(self, session):
        super(SubscriberConnection, self).__init__(session)
        self.session_store = session_store(self)
        self.pub_sub = get_subscription_provider()

    def _close_invalid_origin(self):
        self.close(4000, message='Invalid origin')

    def on_open(self, request):
        if not set_origin_connection(request, self):
            self._close_invalid_origin()
            return

        super(SubscriberConnection, self).on_open(request)
        if is_heartbeat_enabled():
            self.periodic_callback = PeriodicCallback(self.send_heartbeat, get_heartbeat_frequency())
            self.periodic_callback.start()

    def send_heartbeat(self):
        self.send({'heartbeat': '1'})

    def on_heartbeat(self):
        self.session_store.refresh_all_keys()

    def on_close(self):
        if hasattr(self, 'periodic_callback'):
            self.periodic_callback.stop()
        self.pub_sub.close(self)

    def on_message(self, data):
        if not test_origin(self):
            self._close_invalid_origin()

        try:
            data = self.to_json(data)
            if data == {'heartbeat': '1'}:
                self.on_heartbeat()
                return
            handler = route_handler.get_route_handler(data['route'])
            handler(self).handle(data)
        except Exception as e:
            self.abort_connection()
            raise e

    def abort_connection(self):
        self.close(code=3001, message='Connection aborted')

    def close(self, code=3000, message='Connection closed'):
        self.session.close(code, message)


class DjangoSubscriberConnection(SubscriberConnection):
    pass



from swampdragon.route_handler import BaseRouter
from swampdragon.testing.dragon_testcase import DragonTestCase
from swampdragon.pubsub_providers.data_publisher import publish_data


class FooRouter(BaseRouter):
    def get_subscription_channels(self, **kwargs):
        return ['foo-chan']


class TestDataPublisher(DragonTestCase):
    def setUp(self):
        self.router = FooRouter(self.connection)
        self.router.subscribe(**{'channel': 'chan'})

    def test_publish_string(self):
        publish_data('foo-chan', 'hello')
        self.assertEqual(self.connection.last_pub['data'], 'hello')

    def test_publish_dict(self):
        publish_data('foo-chan', {'key': 'value'})
        self.assertEqual(self.connection.last_pub['data']['key'], 'value')



import json


subscribers = {}


class MockPublisher(object):
    def __init__(self):
        self.subscribers = subscribers

    def publish(self, channel, message):
        subs = subscribers.get(channel)
        if not subs:
            return
        for subscriber in subs:
            if isinstance(message, str):
                message = json.dumps(message)
            subscriber.published_data.append(message)

    def _get_channels_from_subscriptions(self, base_channel):
        channels = [key for key in self.subscribers.keys() if key.startswith(base_channel)]
        return channels

    def get_channels(self, base_channel):
        return self._get_channels_from_subscriptions(base_channel)

    def subscribe(self, channels, subscriber):
        for c in channels:
            if c not in subscribers.keys():
                subscribers[c] = []
            subscribers[c].append(subscriber)

    def unsubscribe(self, channels, subscriber):
        if not isinstance(channels, list):
            return self.unsubscribe([channels], subscriber)
        for channel in channels:
            subscribers[channel].remove(subscriber)

        empty_channels = [k for (k, v) in subscribers.items() if not v]
        for k in empty_channels:
            del subscribers[k]

    def remove_subscriber(self, subscriber):
        channels = [c for c in subscribers if subscriber in subscribers[c]]
        self.unsubscribe(channels, subscriber)



from django.db.models import ForeignKey
from .pubsub_providers.base_provider import PUBACTIONS
from .model_tools import get_property
from .pubsub_providers.model_publisher import publish_model
from .serializers.serializer_importer import get_serializer
from django.db.models.signals import pre_delete, m2m_changed
from django.dispatch.dispatcher import receiver


class SelfPublishModel(object):
    serializer_class = None

    def __init__(self, *args, **kwargs):
        if isinstance(self.serializer_class, str):
            self.serializer_class = get_serializer(self.serializer_class, self)
        self._pre_save_state = dict()
        super(SelfPublishModel, self).__init__(*args, **kwargs)
        self._serializer = self.serializer_class(instance=self)
        self._set_pre_save_state()

    def _set_pre_save_state(self):
        """
        Set the state of the model before any changes are done,
        so it's possible to determine what fields have changed.
        """
        relevant_fields = self._get_relevant_fields()
        for field in relevant_fields:
            val = get_property(self, field)
            if hasattr(self._serializer, field):
                continue
            if val is None:
                self._pre_save_state[field] = None
                continue
            self._pre_save_state[field] = val

    def _get_relevant_fields(self):
        """
        Get all fields that will affect the state.
        This is used to save the state of the model before it's updated,
        to be able to get changes used when publishing an update (so not all fields are published)
        """
        relevant_fields = self._serializer.base_fields

        if 'id' in relevant_fields:
            relevant_fields.remove('id')

        relevant_fields_copy = []

        # check for any foreign keys and include the key to the object instead of the object itself
        # This is to avoid triggering queries to retrieve the foreign object
        for field in self._meta.fields:
            if field.name in relevant_fields:
                if type(field) in (ForeignKey, ):
                    # typically field.attname is field.name + "_id" ie field_id
                    relevant_fields_copy.append(field.attname)
                else:
                    relevant_fields_copy.append(field.name)

        return relevant_fields_copy

        # for field_name in relevant_fields:
        #     field = self._meta.get_field_by_name(field_name)[0]
        #     if isinstance(field, ForeignKey):
        #         relevant_fields.remove(field_name)
        #
        # return relevant_fields

    def get_changed_fields(self):
        changed_fields = []
        for k, v in self._pre_save_state.items():
            val = get_property(self, k)
            if val != v:
                changed_fields.append(k)
        return changed_fields

    def serialize(self):
        return self._serializer.serialize()

    def _publish(self, action, changed_fields=None):
        publish_model(self, self._serializer, action, changed_fields)

    def save(self, *args, **kwargs):
        if not self.pk:
            self.action = PUBACTIONS.created
            self.changed_fields = None
        else:
            self.action = PUBACTIONS.updated
            self.changed_fields = self.get_changed_fields()
        super(SelfPublishModel, self).save(*args, **kwargs)
        self._publish(self.action, self.changed_fields)

        # Set the pre-save state to the current state
        # in case the model is changed again before retrieval
        self._set_pre_save_state()


@receiver(m2m_changed)
def _self_publish_model_m2m_change(sender, instance, action, model, pk_set, **kwargs):
    if not isinstance(instance, SelfPublishModel):
        return
    instance.action = PUBACTIONS.updated
    if action in ['post_add', 'post_clear', 'post_remove']:
        instance._publish(instance.action, instance._serializer.opts.publish_fields)


@receiver(pre_delete)
def _self_publish_model_delete(sender, instance, **kwargs):
    if isinstance(instance, SelfPublishModel):
        instance._publish(PUBACTIONS.deleted)



from django import template
from django.conf import settings

register = template.Library()


@register.simple_tag()
def swampdragon_settings():
    root_url = getattr(settings, 'DRAGON_URL') or 'http://localhost:9999/'
    if not root_url.endswith('/'):
        root_url += '/'
    return '<script type="text/javascript" src="{}settings.js"></script>'.format(root_url)



from datetime import datetime
from swampdragon import route_handler
from swampdragon.route_handler import ModelRouter
from swampdragon.testing.dragon_testcase import DragonTestCase
from .models import BarSelfPub, FooSelfPub
from .serializers import BarSelfPubSerializer


class BarRouter(ModelRouter):
    valid_verbs = ['subscribe']
    route_name = 'bar-router'
    model = BarSelfPub
    serializer_class = BarSelfPubSerializer


class TestRelatedModelChange(DragonTestCase):
    def setUp(self):
        super(TestRelatedModelChange, self).setUp()
        route_handler.register(BarRouter)

    def test_alter_related_model(self):
        """
        Changing a self publishing
        """
        foo = FooSelfPub.objects.create(name='test', number=123)
        new_foo = FooSelfPub.objects.create(name='test two', number=456)
        bar = BarSelfPub.objects.create(date=datetime.now(), foo=foo)

        self.connection.subscribe(BarRouter.route_name, 'test')

        self.assertIsNone(self.connection.last_pub)
        bar.foo = new_foo
        bar.save()
        self.assertIsNotNone(self.connection.last_pub)



from django.conf import settings
from ..pubsub_providers import redis_publisher
from .session_store import BaseSessionStore
import json


class RedisSessionStore(BaseSessionStore):
    def __init__(self, connection):
        super(RedisSessionStore, self).__init__(connection)
        self.client = redis_publisher.get_redis_cli()

    def get_complete_key(self, key):
        return 's:{}|k:{}'.format(self.connection.session.session_id, key)

    def set(self, key, val):
        if isinstance(val, dict):
            val = json.dumps(val)
        complete_key = self.get_complete_key(key)
        self.client.set(complete_key, val)
        self.client.expire(complete_key, getattr(settings, 'SESSION_EXPIRATION_TIME', 30) * 60)

    def get(self, key):
        complete_key = self.get_complete_key(key)
        val = self.client.get(complete_key)
        if not val:
            return None
        self.refresh_key_timeout(key)
        return val.decode()

    def refresh_key_timeout(self, key):
        complete_key = self.get_complete_key(key)
        self.client.expire(complete_key, getattr(settings, 'SESSION_EXPIRATION_TIME', 30) * 60)



from swampdragon.default_settings import SwampDragonSettings
from swampdragon.testing.dragon_testcase import DragonTestCase


class TestJavaScriptSettings(DragonTestCase):
    def test_user_settings(self):
        from django.conf import settings as django_settings
        django_settings.SWAMP_DRAGON = {
            'foo': 'bar',
        }
        settings = SwampDragonSettings()
        self.assertEqual(settings.to_dict()['foo'], 'bar')



from tornado.testing import AsyncHTTPTestCase
from swampdragon import discover_routes
from swampdragon import route_handler
from swampdragon.pubsub_providers.subscriber_factory import get_subscription_provider
from swampdragon.connections.mock_connection import TestConnection
from django.test import TestCase
from django.conf import settings
from django.utils.importlib import import_module
from sockjs.tornado import SockJSRouter
from tornado import web
from swampdragon.settings_provider import SettingsHandler


pub_sub = get_subscription_provider()


class DragonTestCase(TestCase):
    def __init__(self, methodName='runTest'):
        super(DragonTestCase, self).__init__(methodName)

        pub_sub._channels = []
        pub_sub._subscribers = {}
        self.connection = TestConnection()
        self.urls = discover_routes()

    def tearDown(self):
        route_handler.registered_handlers = {}
        pub_sub._channels = []
        pub_sub._subscribers = {}

    def _load_app(self):
        routers = []
        module_name, cls_name = settings.SWAMP_DRAGON_CONNECTION[0].rsplit('.', 1)
        module = import_module(module_name)
        cls = getattr(module, cls_name)
        channel = settings.SWAMP_DRAGON_CONNECTION[1]
        routers.append(SockJSRouter(cls, channel))
        print('Channel {}'.format(channel))

        app_settings = {
            'debug': settings.DEBUG,
        }

        urls = discover_routes()
        for router in routers:
            urls += router.urls

        app = web.Application(urls, **app_settings)
        return app


class DragonTestCaseAsync(AsyncHTTPTestCase):
    def __init__(self, methodName='runTest'):
        super(DragonTestCaseAsync, self).__init__(methodName)

        pub_sub._channels = []
        pub_sub._subscribers = {}
        self.connection = TestConnection()
        self.urls = discover_routes()
        self.app = self._load_app()

    def tearDown(self):
        route_handler.registered_handlers = {}
        pub_sub._channels = []
        pub_sub._subscribers = {}

    def _load_app(self):
        routers = []
        module_name, cls_name = settings.SWAMP_DRAGON_CONNECTION[0].rsplit('.', 1)
        module = import_module(module_name)
        cls = getattr(module, cls_name)
        channel = settings.SWAMP_DRAGON_CONNECTION[1]
        routers.append(SockJSRouter(cls, channel))
        print('Channel {}'.format(channel))

        app_settings = {
            'debug': settings.DEBUG,
        }

        urls = discover_routes()
        for router in routers:
            urls += router.urls
        urls.append(('/settings.js$', SettingsHandler))

        app = web.Application(urls, **app_settings)
        return app

    def get_app(self):
        return self.app

    @property
    def host(self):
        return '{}://localhost:{}'.format(self.get_protocol(), self.get_http_port())



from tornado.web import RequestHandler
from swampdragon.default_settings import SwampDragonSettings
from django.conf import settings as django_settings
from .same_origin import set_origin_cookie


def get_host():
    host = django_settings.DRAGON_URL
    if host.endswith('/'):
        return host[:-1]
    return host


class SettingsHandler(RequestHandler):
    def set_default_headers(self):
        self.set_header("Content-Type", "application/javascript")
        set_origin_cookie(self)

    def get(self, *args, **kwargs):
        data = '''window.swampdragon_settings = {settings};
window.swampdragon_host = "{host}";
'''.format(**{
            'settings': SwampDragonSettings().to_dict(),
            'host': get_host()
        })
        self.write(data)



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="SwampDragon",
    version="0.4.2.2",
    author="Jonas Hagstedt",
    author_email="hagstedt@gmail.com",
    description=("SwampDragon is a powerful platform making it easy to build real time web applications, combining the power of Django and Tornado"),
    license="BSD",
    keywords="SwampDragon, websockets, realtime, sockjs, django, tornado, framework",
    url="http://swampdragon.net",
    packages=find_packages(),
    long_description=read('README.txt'),
    include_package_data=True,
    entry_points={'console_scripts': ['dragon-admin = swampdragon.core:run', ]},
    install_requires=[
        "Django >= 1.6, < 1.9",
        "Tornado >= 3.2.2",
        "sockjs-tornado >= 1.0.0",
        "tornado-redis >= 2.4.18",
        "redis >= 2.8",
        "python-dateutil >= 2.2"
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
    ],
)



class ValidationError(Exception):
    def __init__(self, errors={}, *args, **kwargs):
        super(ValidationError, self).__init__(*args, **kwargs)
        self.errors = errors

    def get_error_dict(self):
        return self.errors


class ModelValidationError(ValidationError):
    pass
    # def __init__(self, errors={}, *args, **kwargs):
    #     super(ModelValidationError, self).__init__(*args, **kwargs)
    #     self.errors = errors
    #
    # def get_error_dict(self):
    #     return self.errors



from django.conf import settings
import uuid


SAME_ORIGIN_COOKIE_NAME = 'sdso'


def _origin_required():
    return getattr(settings, 'SWAMP_DRAGON_SAME_ORIGIN', False)


def test_origin(connection):
    if not _origin_required():
        return True

    if not hasattr(connection, SAME_ORIGIN_COOKIE_NAME):
        return False

    return True


def set_origin_cookie(request_handler):
    request_handler.set_cookie(SAME_ORIGIN_COOKIE_NAME, uuid.uuid4().hex)


def set_origin_connection(request, connection):
    if not _origin_required():
        return True

    so_cookie = request.get_cookie(SAME_ORIGIN_COOKIE_NAME)
    if not so_cookie:
        return False

    if not hasattr(connection, SAME_ORIGIN_COOKIE_NAME):
        setattr(connection, SAME_ORIGIN_COOKIE_NAME, so_cookie.value)

    return True
